@@ -124,6 +124,10 @@
 #define EC2_IAM_META_DATA_ACCESSKEY "SecretAccessKey"
 #define EC2_IAM_META_DATA_TOKEN     "Token"
 
+/* TCP keep-alive */
+#define TCP_KEEP_ALIVE_IDLE         200
+#define TCP_KEEP_ALIVE_INTERVAL     60
+
 /* Misc */
 #define WHITESPACE                  " \t\v\f\r\n"
 
@@ -189,6 +193,7 @@ struct http_io {
     const char          *method;                // HTTP method
     const char          *url;                   // HTTP URL
     struct curl_slist   *headers;               // HTTP headers
+    const char          *sse;                   // Server Side Encryption
     void                *dest;                  // Block data (when reading)
     const void          *src;                   // Block data (when writing)
     s3b_block_t         block_num;              // The block we're reading/writing
@@ -233,6 +238,8 @@ static void http_io_get_mounted_flag_url(char *buf, size_t bufsiz, struct http_i
 static int http_io_add_auth(struct http_io_private *priv, struct http_io *io, time_t now, const void *payload, size_t plen);
 static int http_io_add_auth2(struct http_io_private *priv, struct http_io *io, time_t now, const void *payload, size_t plen);
 static int http_io_add_auth4(struct http_io_private *priv, struct http_io *io, time_t now, const void *payload, size_t plen);
+static size_t url_encode(const char *src, size_t len, char *dst, size_t buflen, int encode_slash);
+static void digest_url_encoded(EVP_MD_CTX* hash_ctx, const char *data, size_t len, int encode_slash);
 
 /* EC2 IAM thread */
 static void *update_iam_credentials_main(void *arg);
@@ -349,12 +356,6 @@ http_io_create(struct http_io_conf *config)
             r = EINVAL;
             goto fail4;
         }
-        if (EVP_CIPHER_block_size(priv->cipher) != EVP_CIPHER_iv_length(priv->cipher)) {
-            (*config->log)(LOG_ERR, "invalid encryption cipher `%s': block size %d != IV length %d",
-              config->encryption, EVP_CIPHER_block_size(priv->cipher), EVP_CIPHER_iv_length(priv->cipher));
-            r = EINVAL;
-            goto fail4;
-        }
         cipher_key_len = EVP_CIPHER_key_length(priv->cipher);
         priv->keylen = config->key_length > 0 ? config->key_length : cipher_key_len;
         if (priv->keylen < cipher_key_len || priv->keylen > sizeof(priv->key)) {
@@ -537,6 +538,7 @@ http_io_list_blocks(struct s3backer_store *s3b, block_list_func_t *callback, voi
 
     /* List blocks */
     do {
+        char url_encoded_prefix[strlen(config->prefix) * 3 + 1];
         const time_t now = time(NULL);
 
         /* Reset XML parser state */
@@ -545,16 +547,19 @@ http_io_list_blocks(struct s3backer_store *s3b, block_list_func_t *callback, voi
         XML_SetElementHandler(io.xml, http_io_list_elem_start, http_io_list_elem_end);
         XML_SetCharacterDataHandler(io.xml, http_io_list_text);
 
+        /* URL-encode prefix */
+        url_encode(config->prefix, strlen(config->prefix), url_encoded_prefix, sizeof(url_encoded_prefix), 1);
+
         /* Format URL */
         snprintf(urlbuf, sizeof(urlbuf), "%s%s?", config->baseURL, config->vhost ? "" : config->bucket);
 
         /* Add URL parameters (note: must be in "canonical query string" format for proper authentication) */
         if (io.list_truncated) {
             snprintf(urlbuf + strlen(urlbuf), sizeof(urlbuf) - strlen(urlbuf), "%s=%s%0*jx&",
-              LIST_PARAM_MARKER, config->prefix, S3B_BLOCK_NUM_DIGITS, (uintmax_t)io.last_block);
+              LIST_PARAM_MARKER, url_encoded_prefix, S3B_BLOCK_NUM_DIGITS, (uintmax_t)io.last_block);
         }
         snprintf(urlbuf + strlen(urlbuf), sizeof(urlbuf) - strlen(urlbuf), "%s=%u", LIST_PARAM_MAX_KEYS, LIST_BLOCKS_CHUNK);
-        snprintf(urlbuf + strlen(urlbuf), sizeof(urlbuf) - strlen(urlbuf), "&%s=%s", LIST_PARAM_PREFIX, config->prefix);
+        snprintf(urlbuf + strlen(urlbuf), sizeof(urlbuf) - strlen(urlbuf), "&%s=%s", LIST_PARAM_PREFIX, url_encoded_prefix);
 
         /* Add Date header */
         http_io_add_date(priv, &io, now);
@@ -806,6 +811,7 @@ http_io_set_mounted(struct s3backer_store *s3b, int *old_valuep, int new_value)
     io.url = urlbuf;
     io.method = HTTP_HEAD;
 
+
     /* Construct URL for the mounted flag */
     http_io_get_mounted_flag_url(urlbuf, sizeof(urlbuf), config);
 
@@ -847,6 +853,7 @@ http_io_set_mounted(struct s3backer_store *s3b, int *old_valuep, int new_value)
         io.url = urlbuf;
         io.method = new_value ? HTTP_PUT : HTTP_DELETE;
 
+
         /* Add Date header */
         http_io_add_date(priv, &io, now);
 
@@ -855,7 +862,7 @@ http_io_set_mounted(struct s3backer_store *s3b, int *old_valuep, int new_value)
             struct tm tm;
 
             /* Create content for the mounted flag object (timestamp) */
-            gethostname(content, sizeof(content - 1));
+            gethostname(content, sizeof(content) - 1);
             content[sizeof(content) - 1] = '\0';
             strftime(content + strlen(content), sizeof(content) - strlen(content), "\n" AWS_DATE_BUF_FMT "\n", gmtime_r(&now, &tm));
             io.src = content;
@@ -882,6 +889,10 @@ http_io_set_mounted(struct s3backer_store *s3b, int *old_valuep, int new_value)
         if (storage_class != NULL)
             io.headers = http_io_add_header(io.headers, "%s: %s", STORAGE_CLASS_HEADER, storage_class);
 
+        /* Add Server Side Encryption value (if needed) */
+        if (config->sse != NULL)
+            io.headers = http_io_add_header(io.headers, "%s: %s", S3BACKER_SSE_HEADER, config->sse);
+
         /* Add Authorization header */
         if ((r = http_io_add_auth(priv, &io, now, io.src, io.buf_size)) != 0)
             goto done;
@@ -1124,6 +1135,8 @@ http_io_read_block(struct s3backer_store *const s3b, s3b_block_t block_num, void
     did_read = io.buf_size - io.bufs.rdremain;
 
     /* Check Content-Encoding and decode if necessary */
+    if (*io.content_encoding == '\0' && config->default_ce != NULL)
+        snprintf(io.content_encoding, sizeof(io.content_encoding), "%s", config->default_ce);
     for ( ; r == 0 && *io.content_encoding != '\0'; *layer = '\0') {
 
         /* Find next encoding layer */
@@ -1529,6 +1542,12 @@ http_io_write_block(struct s3backer_store *const s3b, s3b_block_t block_num, con
     if (storage_class != NULL)
         io.headers = http_io_add_header(io.headers, "%s: %s", STORAGE_CLASS_HEADER, storage_class);
 
+    /* Add Server Side Encryption header (if needed) */
+    if (config->sse != NULL) {
+        io.headers = http_io_add_header(io.headers, "%s: %s", S3BACKER_SSE_HEADER, config->sse);
+        (*config->log)(LOG_INFO, "setting SSE header %s to %s", S3BACKER_SSE_HEADER, config->sse);
+    }
+
     /* Add Authorization header */
     if ((r = http_io_add_auth(priv, &io, now, io.src, io.buf_size)) != 0)
         goto fail;
@@ -2060,7 +2079,7 @@ http_io_add_auth4(struct http_io_private *priv, struct http_io *const io, time_t
 #endif
 
     /* Canonical URI */
-    EVP_DigestUpdate(hash_ctx, (const u_char *)uripath, uripath_len);
+    digest_url_encoded(hash_ctx, uripath, uripath_len, 0);
     EVP_DigestUpdate(hash_ctx, (const u_char *)"\n", 1);
 #if DEBUG_AUTHENTICATION
     snprintf(sigbuf + strlen(sigbuf), sizeof(sigbuf) - strlen(sigbuf), "%.*s\n", (int)uripath_len, uripath);
@@ -2241,6 +2260,44 @@ fail:
     return r;
 }
 
+/*
+ * Add data to digest, but in URL-encoded form.
+ */
+static void
+digest_url_encoded(EVP_MD_CTX* hash_ctx, const char *data, size_t len, int encode_slash)
+{
+    char buf[len * 3 + 1];
+
+    len = url_encode(data, len, buf, sizeof(buf), encode_slash);
+    EVP_DigestUpdate(hash_ctx, (const u_char *)buf, len);
+}
+
+/*
+ * URL-encode the given input.
+ */
+static size_t
+url_encode(const char *src, size_t len, char *dst, size_t buflen, int encode_slash)
+{
+    char *const dst_base = dst;
+    size_t elen;
+
+    while (len-- > 0) {
+        const char ch = *src++;
+        if (isalnum(ch) || ch == '_' || ch == '-' || ch == '~' || ch == '.' || (ch == '/' && !encode_slash)) {
+            snprintf(dst, buflen, "%c", ch);
+            elen = 1;
+        } else {
+            snprintf(dst, buflen, "%%%02X", (int)ch & 0xff);
+            elen = 3;
+        }
+        dst += elen;
+        buflen -= elen;
+    }
+    if (buflen > 0)
+        *dst = '\0';
+    return dst - dst_base;
+}
+
 /*
  * Create URL for a block, and return pointer to the URL's URI path.
  */
@@ -2338,6 +2395,9 @@ http_io_acquire_curl(struct http_io_private *priv, struct http_io *io)
     curl_easy_setopt(curl, CURLOPT_FAILONERROR, 1);
     curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);
     curl_easy_setopt(curl, CURLOPT_NOSIGNAL, (long)1);
+    curl_easy_setopt(curl, CURLOPT_TCP_KEEPALIVE, (long)1);
+    curl_easy_setopt(curl, CURLOPT_TCP_KEEPIDLE, (long)TCP_KEEP_ALIVE_IDLE);
+    curl_easy_setopt(curl, CURLOPT_TCP_KEEPINTVL, (long)TCP_KEEP_ALIVE_INTERVAL);
     curl_easy_setopt(curl, CURLOPT_TIMEOUT, (long)config->timeout);
     curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 1);
     curl_easy_setopt(curl, CURLOPT_USERAGENT, config->user_agent);
