@@ -47,6 +47,7 @@
 #define STORAGE_CLASS_STANDARD_IA           "STANDARD_IA"
 #define STORAGE_CLASS_REDUCED_REDUNDANCY    "REDUCED_REDUNDANCY"
 
+
 /* Configuration info structure for http_io store */
 struct http_io_conf {
     char                *accessId;
@@ -64,6 +65,7 @@ struct http_io_conf {
     const char          *cacert;
     const char          *password;
     const char          *encryption;
+    const char          *default_ce;
     u_int               key_length;
     int                 debug;
     int                 debug_http;
@@ -80,6 +82,7 @@ struct http_io_conf {
     u_int               max_retry_pause;
     uintmax_t           max_speed[2];
     log_func_t          *log;
+    const char          *sse;
 };
 
 /* Statistics structure for http_io store */
