@@ -82,6 +82,9 @@
 #define S3BACKER_DEFAULT_COMPRESSION                Z_NO_COMPRESSION
 #define S3BACKER_DEFAULT_ENCRYPTION                 "AES-128-CBC"
 
+#define S3BACKER_SSE_HEADER                         "x-amz-server-side-encryption"
+#define S3BACKER_DEFAULT_SSE_VALUE                  "AES256"
+
 /* MacFUSE setting for kernel daemon timeout */
 #ifdef __APPLE__
 #ifndef FUSE_MAX_DAEMON_TIMEOUT
@@ -157,6 +160,7 @@ static struct s3b_config config = {
         .timeout=               S3BACKER_DEFAULT_TIMEOUT,
         .initial_retry_pause=   S3BACKER_DEFAULT_INITIAL_RETRY_PAUSE,
         .max_retry_pause=       S3BACKER_DEFAULT_MAX_RETRY_PAUSE,
+        .sse =                  NULL
     },
 
     /* "Eventual consistency" protection config */
@@ -238,6 +242,10 @@ static const struct fuse_opt option_list[] = {
         .templ=     "--region=%s",
         .offset=    offsetof(struct s3b_config, http_io.region),
     },
+    {
+            .templ=     "--sse=%s",
+            .offset=    offsetof(struct s3b_config, http_io.sse),
+    },
     {
         .templ=     "--blockCacheSize=%u",
         .offset=    offsetof(struct s3b_config, block_cache.cache_size),
@@ -364,6 +372,10 @@ static const struct fuse_opt option_list[] = {
         .templ=     "--prefix=%s",
         .offset=    offsetof(struct s3b_config, http_io.prefix),
     },
+    {
+        .templ=     "--defaultContentEncoding=%s",
+        .offset=    offsetof(struct s3b_config, http_io.default_ce),
+    },
     {
         .templ=     "--readOnly",
         .offset=    offsetof(struct s3b_config, fuse_ops.read_only),
@@ -1004,6 +1016,13 @@ validate_config(void)
         return -1;
     }
 
+    /* Check storage class */
+    if (config.http_io.sse != NULL
+        && strcmp(config.http_io.sse, S3BACKER_DEFAULT_SSE_VALUE) != 0) {
+        warnx("invalid sse type `%s'", config.http_io.sse);
+        config.http_io.sse = S3BACKER_DEFAULT_SSE_VALUE;
+    }
+
     /* Set default or custom region */
     if (config.http_io.region == NULL)
         config.http_io.region = S3BACKER_DEFAULT_REGION;
@@ -1541,6 +1560,8 @@ dump_config(void)
     (*config.log)(LOG_DEBUG, "%24s: \"%s\"", "region", config.http_io.region);
     (*config.log)(LOG_DEBUG, "%24s: \"%s\"", config.test ? "testdir" : "bucket", config.http_io.bucket);
     (*config.log)(LOG_DEBUG, "%24s: \"%s\"", "prefix", config.http_io.prefix);
+    (*config.log)(LOG_DEBUG, "%24s: \"%s\"", "defaultContentEncoding",
+      config.http_io.default_ce != NULL ? config.http_io.default_ce : "(none)");
     (*config.log)(LOG_DEBUG, "%24s: %s", "list_blocks", config.list_blocks ? "true" : "false");
     (*config.log)(LOG_DEBUG, "%24s: \"%s\"", "mount", config.mount);
     (*config.log)(LOG_DEBUG, "%24s: \"%s\"", "filename", config.fuse_ops.filename);
@@ -1703,6 +1724,7 @@ usage(void)
     fprintf(stderr, "\t--%-27s %s\n", "password=PASSWORD", "Encrypt using PASSWORD");
     fprintf(stderr, "\t--%-27s %s\n", "passwordFile=FILE", "Encrypt using password read from FILE");
     fprintf(stderr, "\t--%-27s %s\n", "prefix=STRING", "Prefix for resource names within bucket");
+    fprintf(stderr, "\t--%-27s %s\n", "defaultContentEncoding=STRING", "Default HTTP Content-Encoding if none given");
     fprintf(stderr, "\t--%-27s %s\n", "quiet", "Omit progress output at startup");
     fprintf(stderr, "\t--%-27s %s\n", "readAhead=NUM", "Number of blocks to read-ahead");
     fprintf(stderr, "\t--%-27s %s\n", "readAheadTrigger=NUM", "# of sequentially read blocks to trigger read-ahead");
